#include <project.h>


#define TIMER_CLK (24000000)
#define SMP_FREQ (48000)
#define SMP_PER_PACKET (SMP_FREQ/1000)

#define TIMER_PERIOD (TIMER_CLK/1000)

extern int16 sin_table[1024];
int16 dmaBuf1[SMP_PER_PACKET];
int16 dmaBuf2[SMP_PER_PACKET];
uint8 sampleNeeded = 0;
volatile int selectedBuf = 1;
#define AUDIO_INTERFACE (1u)
uint16 newFreq = 1000;
uint16 freq = 1000;
uint8 audio_on = 0;

#define min(a,b) (a>b?b:a)
uint32 avgSum = 0;
uint8 avgN = 0;
CY_ISR(AdcIrq)
{
    uint16 adcValue = ADC_GetResult16();
    avgSum += adcValue;
    avgN++;
    if(avgN == 5)
    {        
        avgSum /= 5;
        avgSum -= min(0x68, avgSum); //fix offset
        uint32 temp = ((uint32)avgSum * 20000UL)/0xFFF;
        newFreq = temp + 20; //minimum 20Hz
        avgSum = 0;
        avgN = 0;
    }
}
CY_ISR(TimerIrqHandler)
{
    ADC_StartConvert();
}

//2PI = 4096
uint16 get_sin_value(uint16 index)
{
    if(index >= 4096)
        index %= 4096;
    if(index < 1024)
        return sin_table[index];
    else if(index < 2048)
        return sin_table[2047 - index];
    else if(index < 3072)
        return -sin_table[index - 2048];
    else
        return -sin_table[4095 - index];
}

uint16 get_samples(int16* buf, uint16 count, uint16 startpos, uint32 f, uint32 fs)
{
    int i;
    //fmax = 24000
    //fsmax = 48000
    f <<= 16;
    uint32 smpdelta = f/fs;
    //smpdelta=32768 => sindelta=pi (2048)
    uint32 phi = (smpdelta * startpos + (1 << 3)) >> 4;
    if(smpdelta < (1 << 3))
    {
        smpdelta = (1 << 3);
    }
    uint16 sindelta = (uint16)((smpdelta + (1 << 3)) >> 4);
    for(i = 0; i < count; ++i)
    {
        buf[i] = get_sin_value((uint16)(phi & 0xFFFF));
        phi += sindelta;
        startpos++;
        if(startpos >= fs)
        {
            startpos %= fs;
        }
    }
    return startpos;
}

void lcd_update()
{
    LCD_Position(1,0);
    if(audio_on)
        LCD_PrintString("ON [");
    else    
        LCD_PrintString("OFF[");
    LCD_PrintDecUint16(freq);
    LCD_PrintString("Hz]    ");
}

int main()
{
    CyGlobalIntEnable;  /* enable global interrupts. */
    size_t index = 0;
    ADC_Start();
    ADC_IRQ_StartEx(AdcIrq);
    LCD_Start();
    LCD_PrintString("Rebell Systems");
    timerIrq_StartEx(TimerIrqHandler);
    Timer_WritePeriod(TIMER_PERIOD);
    lcd_update();
    index = get_samples(dmaBuf1, SMP_PER_PACKET, index, freq, SMP_FREQ);
    
    USB_Start(0, USB_3V_OPERATION);
    /* Wait for Device to enumerate */
    while(!USB_GetConfiguration());
    for(;;)
    {
         if(USB_IsConfigurationChanged() != 0u) 
        {
            /* Check the active Alternative setting when configuration changed */
            if((USB_GetConfiguration() != 0u) && (USB_GetInterfaceSetting(AUDIO_INTERFACE) != 0u))
            {   /* Alternate 1 with AudioStreaming interface has been set */
                Timer_Start();
                audio_on = 1;
                lcd_update();
            }    
            else    /* Zero Alternate setting for mute */
            {
                audio_on = 0;
                lcd_update();
                Timer_Stop();
            }
        }
        if (USB_GetEPState(USB_EP2) == USB_IN_BUFFER_EMPTY)
        {
            if(selectedBuf == 1)
            {
                led_1_Write(1);
                USB_LoadInEP(USB_EP2, (uint8*)dmaBuf1, SMP_PER_PACKET * sizeof(dmaBuf2[0]));
                selectedBuf = 2;
                led_1_Write(0);
                index = get_samples(dmaBuf2, SMP_PER_PACKET, index, freq, SMP_FREQ);
            }
            else
            {
                led_2_Write(1);
                USB_LoadInEP(USB_EP2, (uint8*)dmaBuf2, SMP_PER_PACKET * sizeof(dmaBuf2[0]));
                selectedBuf = 1;
                led_2_Write(0);
                index = get_samples(dmaBuf1, SMP_PER_PACKET, index, freq, SMP_FREQ);
            }
        }
        if(newFreq != freq)
        {
            freq = newFreq;
            lcd_update();
        }
    }
}

/* [] END OF FILE */
